@@ -18,7 +18,6 @@ def emacs_bindings(registry, cli_ref):
     search_line = cli_ref().lines['search']
     handle = create_handle_decorator(registry, line)
 
-
     @handle(Keys.ControlA, in_mode=InputMode.INSERT)
     @handle(Keys.ControlA, in_mode=InputMode.SELECTION)
     def _(event):
@@ -114,13 +113,15 @@ def emacs_bindings(registry, cli_ref):
     def _(event):
         """
         """
-        if event._arg == None:
+        if event._arg is None:
             event.append_to_arg_count('-')
 
     @handle(Keys.Escape, Keys.ControlJ, in_mode=InputMode.INSERT)
     @handle(Keys.Escape, Keys.ControlM, in_mode=InputMode.INSERT)
     @handle(Keys.Escape, Keys.ControlJ, in_mode=InputMode.COMPLETE)
     @handle(Keys.Escape, Keys.ControlM, in_mode=InputMode.COMPLETE)
+    @handle(Keys.Escape, Keys.ControlJ, in_mode=InputMode.INCREMENTAL_SEARCH)
+    @handle(Keys.Escape, Keys.ControlM, in_mode=InputMode.INCREMENTAL_SEARCH)
     def _(event):
         """
         Meta + Newline: always accept input.
@@ -147,7 +148,7 @@ def emacs_bindings(registry, cli_ref):
             deleted = line.delete_before_cursor(count=-pos)
             line.set_clipboard(ClipboardData(deleted))
 
-    @handle(Keys.Escape, 'a')
+    @handle(Keys.Escape, 'a', in_mode=InputMode.INSERT)
     def _(event):
         """
         Previous sentence.
@@ -165,7 +166,7 @@ def emacs_bindings(registry, cli_ref):
             words = line.document.text_after_cursor[:pos]
             line.insert_text(words.title(), overwrite=True)
 
-    @handle(Keys.Escape, 'e')
+    @handle(Keys.Escape, 'e', in_mode=InputMode.INSERT)
     def _(event):
         """ Move to end of sentence. """
         # TODO:
@@ -191,21 +192,12 @@ def emacs_bindings(registry, cli_ref):
         if pos:
             line.cursor_position += pos
 
-    #@handle(Keys.Escape, 'b')
-    #def _(event):
-    #    """
-    #    Delete the Word after the cursor. (Delete until end of word.)
-    #    """
-    #    pos = line.document.find_next_word_ending()
-    #    data = ClipboardData(line.delete(pos))
-    #    line.set_clipboard(data)
-
     @handle(Keys.Escape, 'l', in_mode=InputMode.INSERT)
     def _(event):
         """
         Lowercase the current (or following) word.
         """
-        for i in range(event.arg): # XXX: not DRY: see meta_c and meta_u!!
+        for i in range(event.arg):  # XXX: not DRY: see meta_c and meta_u!!
             pos = line.document.find_next_word_ending()
             words = line.document.text_after_cursor[:pos]
             line.insert_text(words.lower(), overwrite=True)
