@@ -19,11 +19,13 @@ from pygments.token import Token
 _tf = Terminal256Formatter()
 
 __all__ = (
+    'Point',
     'Renderer',
+    'Screen',
 )
 
-_Point = namedtuple('Point', 'y x')
-_Size = namedtuple('Size', 'rows columns')
+Point = namedtuple('Point', 'y x')
+Size = namedtuple('Size', 'rows columns')
 
 
 #: If True: write the output of the renderer also to the following file. This
@@ -32,6 +34,19 @@ _Size = namedtuple('Size', 'rows columns')
 _DEBUG_RENDER_OUTPUT = False
 _DEBUG_RENDER_OUTPUT_FILENAME = '/tmp/prompt-toolkit-render-output'
 
+#: Cache for wcwidth sizes.
+_CHAR_SIZES_CACHE = [wcwidth(six.unichr(i)) for i in range(0, 64000)]
+
+
+def _get_width(c):
+    """
+    Return width of character. Wrapper around ``wcwidth``.
+    """
+    try:
+        return _CHAR_SIZES_CACHE[ord(c)]
+    except IndexError:
+        return wcwidth(c)
+
 
 class TerminalCodes:
     """
@@ -60,7 +75,6 @@ class TerminalCodes:
     DISABLE_AUTOWRAP = '\x1b[?7l'
     ENABLE_AUTOWRAP = '\x1b[?7h'
 
-
     @staticmethod
     def CURSOR_GOTO(row=0, column=0):
         """ Move cursor position. """
@@ -80,7 +94,9 @@ class TerminalCodes:
         if amount == 0:
             return ''
         elif amount == 1:
-            return '\x1b[B' # XXX: would just `\n' also do?
+            # Note: Not the same as '\n', '\n' can cause the window content to
+            #       scroll.
+            return '\x1b[B'
         else:
             return '\x1b[%iB' % amount
 
@@ -98,7 +114,7 @@ class TerminalCodes:
         if amount == 0:
             return ''
         elif amount == 1:
-            return '\b' # '\x1b[D'
+            return '\b'  # '\x1b[D'
         else:
             return '\x1b[%iD' % amount
 
@@ -110,7 +126,7 @@ class Char(object):
     # we should display them as follows:
     # Usually this happens after a "quoted insert".
     display_mappings = {
-        '\x1b': '^[', # Escape
+        '\x00': '^@',  # Control space
         '\x01': '^A',
         '\x02': '^B',
         '\x03': '^C',
@@ -137,11 +153,11 @@ class Char(object):
         '\x18': '^X',
         '\x19': '^Y',
         '\x1a': '^Z',
-        '\x00': '^@', # Control space
+        '\x1b': '^[',  # Escape
         '\x1c': '^\\',
         '\x1d': '^]',
         '\x1f': '^_',
-        '\x7f': '^?', # Control backspace
+        '\x7f': '^?',  # Control backspace
     }
 
     def __init__(self, char=' ', token=Token, z_index=0):
@@ -157,7 +173,7 @@ class Char(object):
         # We use the `max(0, ...` because some non printable control
         # characters, like e.g. Ctrl-underscore get a -1 wcwidth value.
         # It can be possible that these characters end up in the input text.
-        return max(0, sum([wcwidth(c) for c in self.char]))
+        return max(0, sum([_get_width(c) for c in self.char]))
 
 
 class Screen(object):
@@ -169,15 +185,14 @@ class Screen(object):
     """
     def __init__(self, size):
         self._buffer = defaultdict(lambda: defaultdict(Char))
-        self._cursor_mappings = { } # Map `source_string_index` of input data to (row, col) screen output.
+        self._cursor_mappings = {}  # Map `source_string_index` of input data to (row, col) screen output.
         self._x = 0
         self._y = 0
-        self._max_y = 0
 
         self.size = size
         self._left_margin_func = lambda linenr, is_new_line: None
 
-        self.cursor_position = _Point(y=0, x=0)
+        self.cursor_position = Point(y=0, x=0)
         self._line_number = 1
 
     @property
@@ -198,13 +213,13 @@ class Screen(object):
         self._left_margin_func = func or (lambda linenr, is_new_line: None)
 
     def write_char(self, char, token, string_index=None,
-                    set_cursor_position=False, z_index=False):
+                   set_cursor_position=False, z_index=False):
         """
         Write char to current cursor position and move cursor.
         """
         assert len(char) == 1
 
-        char_obj  = Char(char, token, z_index)
+        char_obj = Char(char, token, z_index)
         char_width = char_obj.width
 
         # In case there is no more place left at this line, go first to the
@@ -214,13 +229,13 @@ class Screen(object):
             self._x = 0
             self._left_margin_func(self._line_number, False)
 
-        insert_pos = self._y, self._x
+        insert_pos = self._y, self._x  # XXX: make a Point of this?
 
         if string_index is not None:
             self._cursor_mappings[string_index] = insert_pos
 
         if set_cursor_position:
-            self.cursor_position = _Point(y=self._y, x=self._x)
+            self.cursor_position = Point(y=self._y, x=self._x)
 
         # Insertion of newline
         if char == '\n':
@@ -231,20 +246,30 @@ class Screen(object):
 
         # Insertion of a 'visible' character.
         else:
-            self.write_at_pos(self._y, self._x, char_obj)
+            if char_obj.z_index >= self._buffer[self._y][self._x].z_index:
+                self._buffer[self._y][self._x] = char_obj
+
+            # When we have a double width character, store this byte in the
+            # second cell. (The highest unicode byte) So that if this character
+            # gets deleted afterwarsd, the ``output_screen_diff`` will notice
+            # that this byte is also gone and redraw both cells.
+            if char_width > 1:
+                self._buffer[self._y][self._x+1] = Char(six.unichr(0x110000-1))
 
             # Move position
             self._x += char_width
 
         return insert_pos
 
-    def write_at_pos(self, y, x, char_obj): #char, token, z_index=0):
+    def write_at_pos(self, y, x, char_obj):
         """
         Write character at position (y, x).
         (Truncate when character is outside margin.)
+
+        :param char_obj: :class:`.Char` instance.
         """
         # Add char to buffer
-        if y < self.size.columns:
+        if x < self.size.columns:
             if char_obj.z_index >= self._buffer[y][x].z_index:
                 self._buffer[y][x] = char_obj
 
@@ -255,8 +280,9 @@ class Screen(object):
         """
         for token, text in data:
             for c in text:
-                self.write_at_pos(y, x, Char(c, token, z_index=z_index))
-                x += wcwidth(c)
+                char_obj = Char(c, token, z_index=z_index)
+                self.write_at_pos(y, x, char_obj)
+                x += char_obj.width
 
     def write_highlighted(self, data):
         """
@@ -272,8 +298,8 @@ def output_screen_diff(screen, current_pos, previous_screen=None, last_char=None
     Create diff of this screen with the previous screen.
     """
     #: Remember the last printed character.
-    last_char = [last_char] # nonlocal
-    background_turned_on = [False] # Nonlocal
+    last_char = [last_char]  # nonlocal
+    background_turned_on = [False]  # Nonlocal
 
     #: Variable for capturing the output.
     result = []
@@ -296,7 +322,7 @@ def output_screen_diff(screen, current_pos, previous_screen=None, last_char=None
             write(TerminalCodes.RESET_ATTRIBUTES)
             write(TerminalCodes.NEWLINE * (new.y - current_pos.y))
             write(TerminalCodes.CURSOR_FORWARD(new.x))
-            last_char[0] = None # Forget last char after resetting attributes.
+            last_char[0] = None  # Forget last char after resetting attributes.
         elif new.y < current_pos.y:
             write(TerminalCodes.CURSOR_UP(current_pos.y - new.y))
 
@@ -341,8 +367,8 @@ def output_screen_diff(screen, current_pos, previous_screen=None, last_char=None
                 bg = _tf._color_index(style['bgcolor']) if style['bgcolor'] else None
 
                 e = EscapeSequence(fg=fg, bg=bg,
-                        bold=style.get('bold', False),
-                        underline=style.get('underline', False))
+                                   bold=style.get('bold', False),
+                                   underline=style.get('underline', False))
 
                 write(TerminalCodes.RESET_ATTRIBUTES)
                 write(e.color_string())
@@ -362,9 +388,9 @@ def output_screen_diff(screen, current_pos, previous_screen=None, last_char=None
         write(TerminalCodes.DISABLE_AUTOWRAP)
         write(TerminalCodes.RESET_ATTRIBUTES)
 
-    # When the previous screen has a different width, redraw everything anyway.
+    # When the previous screen has a different size, redraw everything anyway.
     if not previous_screen or previous_screen.size != screen.size:
-        current_pos = move_cursor(_Point(0, 0))
+        current_pos = move_cursor(Point(0, 0))
         write(TerminalCodes.RESET_ATTRIBUTES)
         write(TerminalCodes.ERASE_DOWN)
 
@@ -376,7 +402,7 @@ def output_screen_diff(screen, current_pos, previous_screen=None, last_char=None
 
     # Loop over the rows.
     row_count = max(screen.current_height, previous_screen.current_height)
-    c = 0 # Column counter.
+    c = 0  # Column counter.
 
     for y, r in enumerate(range(0, row_count)):
         new_row = screen._buffer[r]
@@ -391,7 +417,7 @@ def output_screen_diff(screen, current_pos, previous_screen=None, last_char=None
             char_width = (new_row[c].width or 1)
 
             if not chars_are_equal(new_row[c], previous_row[c]):
-                current_pos = move_cursor(_Point(y=y, x=c))
+                current_pos = move_cursor(Point(y=y, x=c))
                 output_char(new_row[c])
                 current_pos = current_pos._replace(x=current_pos.x + char_width)
 
@@ -399,14 +425,14 @@ def output_screen_diff(screen, current_pos, previous_screen=None, last_char=None
 
         # If the new line is shorter, trim it
         if previous_screen and new_max_line_len < previous_max_line_len:
-            current_pos = move_cursor(_Point(y=y, x=new_max_line_len+1))
+            current_pos = move_cursor(Point(y=y, x=new_max_line_len+1))
             write(TerminalCodes.RESET_ATTRIBUTES)
             write(TerminalCodes.ERASE_END_OF_LINE)
-            last_char[0] = None # Forget last char after resetting attributes.
+            last_char[0] = None  # Forget last char after resetting attributes.
 
     # Move cursor:
     if accept_or_abort:
-        current_pos = move_cursor(_Point(y=current_height, x=0))
+        current_pos = move_cursor(Point(y=current_height, x=0))
         write(TerminalCodes.ERASE_DOWN)
     else:
         current_pos = move_cursor(screen.get_cursor_position())
@@ -427,7 +453,6 @@ def output_screen_diff(screen, current_pos, previous_screen=None, last_char=None
 
 class Renderer(object):
     """
-
     Typical usage:
 
     ::
@@ -450,7 +475,7 @@ class Renderer(object):
 
     def reset(self):
         # Reset position
-        self._cursor_pos = _Point(x=0, y=0)
+        self._cursor_pos = Point(x=0, y=0)
 
         # Remember the last screen instance between renderers. This way,
         # we can create a `diff` between two screens and only output the
@@ -461,18 +486,21 @@ class Renderer(object):
 
     def get_size(self):
         rows, columns = get_size(self._stdout.fileno())
-        return _Size(rows=rows, columns=columns)
+        return Size(rows=rows, columns=columns)
 
     def render_to_str(self, abort=False, accept=False):
-        # Generate the output of the new screen.
+        """
+        Generate the output of the new screen. Return as string.
+        """
         screen = self.screen_cls(size=self.get_size())
 
         self.prompt.write_to_screen(screen, self._last_screen.current_height if self._last_screen else 0,
-                abort=abort, accept=accept)
+                                    abort=abort, accept=accept)
 
-        output, self._cursor_pos, self._last_char = output_screen_diff(screen, self._cursor_pos,
-                self._last_screen, self._last_char, accept or abort,
-                style=self._style, grayed=abort)
+        output, self._cursor_pos, self._last_char = output_screen_diff(
+            screen, self._cursor_pos,
+            self._last_screen, self._last_char, accept or abort,
+            style=self._style, grayed=abort)
         self._last_screen = screen
 
         return output
@@ -508,45 +536,3 @@ class Renderer(object):
         self._stdout.write(TerminalCodes.CURSOR_GOTO(0, 0))
 
         self.reset()
-
-    def _in_columns(self, item_iterator, margin_left=0): # XXX: copy of deployer.console.in_columns
-        """
-        :param item_iterator: An iterable, which yields either ``basestring``
-                              instances, or (colored_item, length) tuples.
-        """
-        # Helper functions for extracting items from the iterator
-        def get_length(item):
-            return len(item) if isinstance(item, six.string_types) else item[1]
-
-        def get_text(item):
-            return item if isinstance(item, six.string_types) else item[0]
-
-        # First, fetch all items
-        all_items = list(item_iterator)
-
-        if not all_items:
-            return
-
-        # Calculate the longest.
-        max_length = max(map(get_length, all_items)) + 1
-
-        # World per line?
-        term_width = self.get_size().columns - margin_left
-        words_per_line = int(max(term_width / max_length, 1))
-
-        # Iterate through items.
-        margin = ' ' * margin_left
-        line = [ margin ]
-        for i, j in enumerate(all_items):
-            # Print command and spaces
-            line.append(get_text(j))
-
-            # When we reached the max items on this line, yield line.
-            if (i+1) % words_per_line == 0:
-                yield ''.join(line)
-                line = [ margin ]
-            else:
-                # Pad with whitespace
-                line.append(' ' * (max_length - get_length(j)))
-
-        yield ''.join(line)
